/*********************************************
* vim: set sw=8 ts=8 si et :
* Author: Guido Socher, Copyright: GPL 
* Atmaga328p software for heating pause relay.
* Turn off relay for 3min every hour.
*
* Clock frequency     : Internal clock 1 Mhz (factory default)
*********************************************/
#include <avr/io.h>
#define F_CPU 1000000UL  // 1 MHz
#include <util/delay.h>
#include <avr/interrupt.h>

// define PC5 as output
#define GLEDDDROUT DDRC|= (1<<DDC5);
// set output to VCC, green LED off
#define GLEDOFF PORTC|=(1<<PORTC5)
// set output to GND, green LED on
#define GLEDON PORTC&=~(1<<PORTC5)
// to test the state of the LED
#define GLEDISOFF PORTC&(1<<PORTC5)

// define PC4 as output
#define R2LEDDDROUT DDRC|= (1<<DDC4);
// set output to VCC, red LED off
#define R2LEDOFF PORTC|=(1<<PORTC4)
// set output to GND, red LED on
#define R2LEDON PORTC&=~(1<<PORTC4)
// to test the state of the LED
#define R2LEDISOFF PORTC&(1<<PORTC4)

// define PC3 as output
#define R1LEDDDROUT DDRC|= (1<<DDC3);
// set output to VCC, red LED off
#define R1LEDOFF PORTC|=(1<<PORTC3)
// set output to GND, red LED on
#define R1LEDON PORTC&=~(1<<PORTC3)
// to test the state of the LED
#define R1LEDISOFF PORTC&(1<<PORTC3)

// define PC2 as output
#define R0LEDDDROUT DDRC|= (1<<DDC2);
// set output to VCC, red LED off
#define R0LEDOFF PORTC|=(1<<PORTC2)
// set output to GND, red LED on
#define R0LEDON PORTC&=~(1<<PORTC2)
// to test the state of the LED
#define R0LEDISOFF PORTC&(1<<PORTC2)

// Relay output PD6 as output
#define RELAYDDROUT DDRD|= (1<<DDD6);
// set output to VCC, Relay on 
#define RELAYON PORTD|=(1<<PORTD6)
// set output to GND, red LED on
#define RELAYOFF PORTD&=~(1<<PORTD6)
// to test the state of the LED
#define RELAYISON PORTD&(1<<PORTD6)

static volatile uint8_t sec=0;
static volatile uint8_t min=0;

// _delay_ms uses a floating point datatype if you call
// that function in many places in your code then it becomes
// very fat. An integer is enough for us:
//
// delay x milliseconds:
void delay_ms(unsigned int xms)
{
        while(xms){
                _delay_ms(0.96);
                xms--;
        }
}
// Generate a 1s clock signal as interrupt form the 1MHz system clock
// Since we have that 1024 prescaler we do not generate exactly one second
// but it's good enough.
void timer_init(void)
{
        /* write high byte first for 16 bit register access: */
        TCNT1H=0;  /* set counter to zero*/
        TCNT1L=0;
        // Mode 4 table 14-4 page 132. CTC mode and top in OCR1A
        // WGM13=0, WGM12=1, WGM11=0, WGM10=0
        TCCR1A=(0<<COM1B1)|(0<<COM1B0)|(0<<WGM11);
        TCCR1B=(1<<CS12)|(1<<CS10)|(1<<WGM12)|(0<<WGM13); // clock/1024

        // At what value to cause interrupt.
        // 1000000/1024=976 = 0x3d0
        // Since we count from zeor we have to subtract one: 0x3 and 0xcf
        OCR1AH=0x3;
        OCR1AL=0xcf;
        // interrupt mask bit:
        TIMSK1 = (1 << OCIE1A);
}


// interrupt, step seconds counter
ISR(TIMER1_COMPA_vect){
        sec++;
        if (sec>59){
                min++;
                sec=0;
        }
}


int main(void)
{
        uint8_t pause=0;
        uint8_t cnt=0; // binary counter connected to red leds, 0-7
        // enable LED outputs:
        GLEDDDROUT;
        R0LEDDDROUT;
        R1LEDDDROUT;
        R2LEDDDROUT;
        R0LEDOFF;
        R1LEDOFF;
        R2LEDOFF;
        // relay is by default on:
        RELAYDDROUT;
        RELAYON;

        // enable PB0, as input, switch goes from PB0 to gnd
        DDRB&= ~(1<<DDB0);
        PORTB|= (1<<PINB0); // internal pullup resistor on

        timer_init();
        sei(); // interrupt on, clock starts ticking now

	while (1) {
		delay_ms(200);
                if (min>59){ // 60min
                        pause=1;
                        cnt++; cnt=cnt&7; // count up, wrap counter after 7
                        min=0;
                }
                if (pause && min>=3){ // end pause after 3min
                        pause=0; 
                }
                // set the red leds according to the binary counter cnt
                if (cnt&1){
                        R0LEDON;
                }else{
                        R0LEDOFF;
                }
                if (cnt&2){
                        R1LEDON;
                }else{
                        R1LEDOFF;
                }
                if (cnt&4){
                        R2LEDON;
                }else{
                        R2LEDOFF;
                }
                if (pause){
                        // blink green led:
                        if (GLEDISOFF){
                                GLEDON;
                        }else{
                                GLEDOFF;
                        }
                        if (bit_is_clear(PINB,PINB0)){ // switch closed
                                RELAYON;
                        }else{
                                RELAYOFF;
                        }
                }else{
                        RELAYON;
                        GLEDON;
                }
        }
        return(0);
}

